# coding=utf-8
"""

"""
from rest_framework import mixins
from rest_framework import viewsets

from crams import permissions
from crams_contact import models
from crams_contact.serializers import contact_role


class ContactRoleViewSet(viewsets.GenericViewSet,
                         mixins.ListModelMixin):
    """
    class ContactRoleViewSet
        Gets all the contact roles for a EResearch Body
    """
    permission_classes = [permissions.IsCramsAuthenticated]
    serializer_class = contact_role.ContactRoleSerializer
    queryset = models.ContactRole.objects.all()

    def get_queryset(self):
        queryset = self.queryset
        if 'erb' in self.request.query_params:
            erb = self.request.query_params['erb']
            queryset = self.queryset.filter(
                e_research_body__name=erb)

        if 'e_research_body' in self.request.query_params:
            erb = self.request.query_params['e_research_body']
            queryset = self.queryset.filter(
                e_research_body__name=erb)

        return queryset



# coding=utf-8
"""
    Contact Config
"""

# Contact ErbId related
ERBS_CONTACT_ID_UPDATE_FN_MAP = dict()

# Contact System Identifier Email Template
CONTACT_SYSTEM_IDENTIFIER_EMAIL_TEMPLATE = dict()


def update_erb_contact_ids_for_key(contact_obj):
    for id_key_name, fn in ERBS_CONTACT_ID_UPDATE_FN_MAP.items():
        fn(contact_obj, id_key_name)



from crams_contact.models import Department, Faculty, Organisation
from crams.serializers import model_serializers
from rest_framework import serializers, exceptions
from django.db.models import Q


# copy from common.serializers.lookup_serializers.py d3_prod branch
class OrganisationLookupSerializer(model_serializers.ModelLookupSerializer):
    id = serializers.IntegerField(required=False)

    name = serializers.CharField(required=False)

    short_name = serializers.CharField(required=False)

    class Meta(object):
        model = Organisation
        fields = ['id', 'name', 'short_name', 'ands_url']
        pk_fields = ['name']

    def validate(self, attrs):
        super().validate(attrs)
        self.verify_field_value('name', attrs, ignore_case=True)
        self.verify_field_value('short_name', attrs, ignore_case=True)
        if self.instance:
            return self.instance
        return attrs


class FacultyLookupSerializer(model_serializers.ReadOnlyModelSerializer):
    organisation = serializers.SlugRelatedField(
        slug_field='name', required=True, queryset=Organisation.objects.all())

    class Meta(object):
        model = Faculty
        fields = ('id', 'name', 'organisation')

    def validate(self, data):
        if 'id' not in data:
            name = data.get('name')
            organisation = data.get('organisation')
            msg = 'Error fetching Faculty {} for organisation {}'
            try:
                faculty, created = Faculty.objects.get_or_create(
                    name=name, organisation=organisation)
                return faculty
            except Faculty.DoesNotExist:
                msg = 'Faculty {} does not exist for organisation {}'
            except Faculty.MultipleObjectsReturned:
                msg = 'Faculty {} not unique for organisation {} '

            raise exceptions.ValidationError(msg.format(name, organisation))
        else:
            msg = 'Error fetching Faculty for id {}'
            pk_str = data.get('id')
            try:
                pk = int(pk_str)
                return Faculty.objects.get(pk=pk)
            except ValueError:
                msg = 'Id param must be integer, got {}'
            except Faculty.DoesNotExist:
                msg = 'Faculty does not exist for id {}'

            raise exceptions.ValidationError(msg.format(pk_str))

    @classmethod
    def lookup_obj_qs(cls, data_dict):
        qs_filter = Q()
        if 'name' in data_dict:
            qs_filter &= Q(name__iexact=data_dict.get('name').lower())

        if 'name_contains' in data_dict:
            pattern = data_dict.get('name_contains').lower()
            qs_filter &= Q(name__icontains=pattern)

        if 'organisation' in data_dict:
            org = data_dict.get('organisation')
            if isinstance(org, Organisation):
                qs_filter &= Q(organisation__in=org)
            else:
                qs_filter &= Q(organisation__name__iexact=org.lower())

        return Faculty.objects.filter(qs_filter)


class DepartmentLookupSerializer(model_serializers.ReadOnlyModelSerializer):
    faculty = FacultyLookupSerializer(required=True)

    class Meta(object):
        model = Department
        fields = ('name', 'faculty')

    def validate(self, data):
        msg = 'Error fetching Department {} for {}'
        name = data.get('name')
        faculty = data.get('faculty')
        try:
            department, created = \
                Department.objects.get_or_create(name=name, faculty=faculty)
            return department
        except Department.DoesNotExist:
            msg = 'Department {} does not exist for {}'
        except Department.MultipleObjectsReturned:
            msg = 'Department {} cannot be uniquely determined for {}'

        raise exceptions.ValidationError(msg.format(name, faculty))

    @classmethod
    def lookup_obj_qs(self, data_dict):
        qs_filter = Q()
        if 'name' in data_dict:
            qs_filter &= Q(name__iexact=data_dict.get('name').lower())

        if 'name_contains' in data_dict:
            pattern = data_dict.get('name_contains').lower()
            qs_filter &= Q(name__icontains=pattern)

        if 'faculty' in data_dict:
            faculty = data_dict.get('faculty')
            if isinstance(faculty, Faculty):
                qs_filter &= Q(faculty=faculty)
            elif isinstance(faculty, dict):
                faculty_qs = FacultyLookupSerializer.lookup_obj_qs(faculty)
                if faculty_qs.count() > 0:
                    qs_filter &= Q(faculty__in=faculty_qs)
            else:
                err_msg = 'faculty must be dict or Faculty obj'
                raise exceptions.ValidationError(err_msg)

        return Department.objects.filter(qs_filter)


class DepartmentFlatSerializer(model_serializers.ReadOnlyModelSerializer):
    department_id = serializers.IntegerField(
        source='id', required=False)

    faculty_id = serializers.IntegerField(source='faculty.id', required=False)

    organisation_id = serializers.IntegerField(
        source='faculty.organisation.id', required=False)

    department = serializers.CharField(source='name', required=True)

    faculty = serializers.CharField(source='faculty.name', required=True)

    organisation = serializers.CharField(
        source='faculty.organisation.name', required=True)

    class Meta(object):
        model = Department
        fields = ('department', 'faculty', 'organisation',
                  'organisation_id', 'faculty_id', 'department_id')

    def validate(self, data):
        faculty_dict = data['faculty']
        faculty_dict['organisation'] = faculty_dict['organisation']['name']
        serializer = DepartmentLookupSerializer(data=data)
        serializer.is_valid(raise_exception=True)
        return serializer.validated_data



# Generated by Django 3.1.1 on 2021-02-02 04:03

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('crams_contact', '0002_load_contact_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ServiceManager',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True)),
                ('contact', models.OneToOneField(on_delete=django.db.models.deletion.DO_NOTHING, to='crams_contact.contact')),
            ],
        ),
        migrations.CreateModel(
            name='OrganisationManager',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True)),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='manager_organisations', to='crams_contact.contact')),
                ('organisation', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='organisation_managers', to='crams_contact.organisation')),
            ],
            options={
                'unique_together': {('contact', 'organisation')},
            },
        ),
        migrations.CreateModel(
            name='FacultyManager',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True)),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='manager_faculties', to='crams_contact.contact')),
                ('faculty', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='faculty_managers', to='crams_contact.faculty')),
            ],
            options={
                'unique_together': {('contact', 'faculty')},
            },
        ),
        migrations.CreateModel(
            name='DepartmentManager',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True)),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='manager_departments', to='crams_contact.contact')),
                ('department', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='department_managers', to='crams_contact.department')),
            ],
            options={
                'unique_together': {('contact', 'department')},
            },
        ),
    ]



from json import loads as json_loads

from rest_framework import generics
from rest_framework.exceptions import ParseError
from rest_framework.response import Response
from crams.utils.role import AbstractCramsRoleUtils as roleUtils
from crams.permissions import IsCramsAuthenticated
from crams_contact.serializers import contact_serializer


class CurrentUserRolesView(generics.RetrieveAPIView):
    """
        current user roles view
    """
    permission_classes = [IsCramsAuthenticated]

    @classmethod
    def fetch_crams_token_roles(cls, user):
        if not hasattr(user, 'auth_token'):
            raise ParseError('Current user has no token, hence roles not set')

        json_roles = user.auth_token.cramstoken.ks_roles

        if not json_roles:
            return dict()

        return json_loads(json_roles)

    @classmethod
    def convert_user_roles_for_legacy_display(cls, roles):
        roles_dict = {
            roleUtils.APPROVER_ROLE_KEY: roleUtils.APPROVER_ROLE_KEY in roles,
            roleUtils.PROVIDER_ROLE_KEY: roleUtils.PROVIDER_ROLE_KEY in roles,
            roleUtils.ADMIN_ROLE_KEY: roleUtils.ADMIN_ROLE_KEY in roles,
            roleUtils.SERVICE_MANAGEMENT_ROLE_TYPE:
                roleUtils.SERVICE_MANAGEMENT_ROLE_TYPE in roles,
            roleUtils.FACULTY_MANAGEMENT_ROLE_TYPE:
                roleUtils.FACULTY_MANAGEMENT_ROLE_TYPE in roles
        }
        return roles_dict

    @classmethod
    def fetch_erb_user_roles_boolean(cls, contact_obj):
        def init_erb_role_dict():
            return {
                roleUtils.APPROVER_ROLE_KEY: False,
                roleUtils.PROVIDER_ROLE_KEY: False,
                roleUtils.ADMIN_ROLE_KEY: False,
                roleUtils.TENANT_MANAGER: False
            }

        is_manager = hasattr(contact_obj, 'servicemanager') and \
                     contact_obj.servicemanager.active
        is_faculty = contact_obj.manager_faculties.exists() or \
                     contact_obj.manager_organisations.exists()
        erb_role_dict = {
            roleUtils.SERVICE_MANAGEMENT_ROLE_TYPE: is_manager,
            roleUtils.FACULTY_MANAGEMENT_ROLE_TYPE: is_faculty
        }
        if not hasattr(contact_obj, 'user_roles'):
            return erb_role_dict

        for erb_roles in contact_obj.user_roles.filter(
                end_date_ts__isnull=True):
            role_dict = init_erb_role_dict()
            erb_role_dict[erb_roles.role_erb.name.lower()] = role_dict
            is_admin = erb_roles.is_erb_admin or \
                       erb_roles.admin_erb_systems.exists()
            role_dict[roleUtils.ADMIN_ROLE_KEY] = is_admin
            if is_admin:
                role_dict[roleUtils.APPROVER_ROLE_KEY] = True
                role_dict[roleUtils.PROVIDER_ROLE_KEY] = True
            else:
                role_dict[roleUtils.APPROVER_ROLE_KEY] = \
                    erb_roles.approver_erb_systems.exists() or \
                    erb_roles.delegates.exists()
                role_dict[roleUtils.PROVIDER_ROLE_KEY] = \
                    erb_roles.providers.exists()
            role_dict[roleUtils.TENANT_MANAGER] = erb_roles.is_tenant_manager

        return erb_role_dict

    def get(self, request, *args, **kwargs):
        """

        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        user = request.user
        ret_dict = dict()
        context = {'request': request}
        contact = contact_serializer.ContactSerializer(context=context).fetch_or_create_given_user_as_contact(user)
        ret_dict['contact'] = contact_serializer.ContactSerializer(contact, context=context).data
        # sz = user_id_provision.ProvisionContactIDSerializer
        # ret_dict['contact_ids'] = \
        #     sz(contact.system_identifiers, context=context, many=True).data

        token_roles = self.fetch_crams_token_roles(user)
        ret_dict['crams_token_roles'] = token_roles
        ret_dict['erb_roles'] = self.fetch_erb_user_roles_boolean(contact)
        ret_dict['user_roles'] = {
            'global': self.convert_user_roles_for_legacy_display(token_roles)
        }
        return Response(ret_dict)



# coding=utf-8
"""

"""
from crams.serializers import model_serializers
from crams_contact.models import ContactRole


class ContactRoleSerializer(model_serializers.ReadOnlyModelSerializer):
    """class ContactRoleSerializer."""

    class Meta(object):
        """metaclass."""

        model = ContactRole
        fields = ['id', 'name']



# Generated by Django 3.1.1 on 2021-02-02 03:41

import crams_contact.utils.validators
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('crams', '0008_load_crams_lookups'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=50, null=True)),
                ('given_name', models.CharField(blank=True, max_length=200, null=True)),
                ('surname', models.CharField(blank=True, max_length=200, null=True)),
                ('email', models.EmailField(db_index=True, max_length=254, unique=True)),
                ('phone', models.CharField(blank=True, max_length=50, null=True)),
                ('orcid', models.URLField(blank=True, null=True, validators=[crams_contact.utils.validators.validate_orcid], verbose_name='ORCID of contact')),
                ('scopusid', models.CharField(blank=True, max_length=50, null=True)),
                ('latest_contact', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='crams_contact.contact')),
            ],
        ),
        migrations.CreateModel(
            name='Organisation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('short_name', models.CharField(db_index=True, max_length=50)),
                ('ands_url', models.URLField(blank=True, null=True)),
                ('notification_email', models.EmailField(max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name='Faculty',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('faculty_code', models.CharField(max_length=200)),
                ('name', models.CharField(max_length=200)),
                ('organisation', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='faculties', to='crams_contact.organisation')),
            ],
        ),
        migrations.CreateModel(
            name='EResearchContactIdentifier',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.CharField(max_length=254)),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='archive_contact_ids', to='crams_contact.contact')),
                ('parent_erb_contact_id', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='crams_contact.eresearchcontactidentifier')),
                ('provision_details', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='eresearchcontactidentifier', to='crams.provisiondetails')),
                ('system', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='contact_identifiers', to='crams.eresearchbodyidkey')),
            ],
        ),
        migrations.CreateModel(
            name='Department',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('department_code', models.CharField(max_length=200)),
                ('name', models.CharField(max_length=200)),
                ('faculty', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='departments', to='crams_contact.faculty')),
            ],
        ),
        migrations.CreateModel(
            name='ContactDetail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(choices=[('B', 'Business'), ('P', 'Personal'), ('M', 'Mobile')], default='P', max_length=1)),
                ('restricted', models.BooleanField(default=False)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('phone', models.CharField(blank=True, max_length=50, null=True)),
                ('parent_contact', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='contact_details', to='crams_contact.contact')),
            ],
        ),
        migrations.AddField(
            model_name='contact',
            name='organisation',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='crams_contact.organisation'),
        ),
        migrations.CreateModel(
            name='CramsERBUserRoles',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_erb_admin', models.BooleanField(default=False)),
                ('is_tenant_manager', models.BooleanField(default=False)),
                ('end_date_ts', models.DateTimeField(blank=True, null=True)),
                ('admin_erb_systems', models.ManyToManyField(related_name='admin_contacts', to='crams.EResearchBodySystem')),
                ('approver_erb_systems', models.ManyToManyField(related_name='approver_contacts', to='crams.EResearchBodySystem')),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='user_roles', to='crams_contact.contact')),
                ('delegates', models.ManyToManyField(related_name='delegate_contacts', to='crams.EResearchBodyDelegate')),
                ('providers', models.ManyToManyField(related_name='provider_contacts', to='crams.Provider')),
                ('role_erb', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='contacts', to='crams.eresearchbody')),
            ],
            options={
                'unique_together': {('contact', 'role_erb', 'end_date_ts')},
            },
        ),
        migrations.CreateModel(
            name='ContactRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
                ('project_leader', models.BooleanField(default=False)),
                ('read_only', models.BooleanField(default=False)),
                ('support_notification', models.BooleanField(default=False)),
                ('e_research_body', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='contact_roles', to='crams.eresearchbody')),
            ],
            options={
                'unique_together': {('name', 'e_research_body')},
            },
        ),
        migrations.CreateModel(
            name='ContactLabel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('creation_ts', models.DateTimeField(auto_now_add=True)),
                ('last_modified_ts', models.DateTimeField(auto_now=True)),
                ('label_name', models.CharField(max_length=255)),
                ('active', models.BooleanField(default=True)),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='labels', to='crams_contact.contact')),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='contactlabel_created_by', to=settings.AUTH_USER_MODEL)),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='contactlabel_updated_by', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('label_name', 'contact')},
            },
        ),
    ]



from django.contrib import admin

from crams_contact import models


class ContactDetailAdmin(admin.ModelAdmin):
    """
    class ContactAdmin
    """
    list_display = ('type', 'email', 'phone', 'parent_contact')


class ContactAdmin(admin.ModelAdmin):
    """
    class ContactAdmin
    """
    list_display = ('given_name', 'surname', 'email')


class EResearchContactIdentifierAdmin(admin.ModelAdmin):
    list_display = ('identifier', 'contact',
                    'system', 'parent_erb_contact_id', 'provision_details')
    list_filter = ['provision_details__status', 'system', 'contact']


class OrganisationAdmin(admin.ModelAdmin):
    """
    class OrganisationAdmin
    """
    list_display = ('name', 'short_name', 'ands_url')


class ContactRoleAdmin(admin.ModelAdmin):
    """
    class ContactRoleAdmin
    """
    list_display = ('name', 'project_leader', 'e_research_body', 'read_only')
    list_filter = ['e_research_body', 'project_leader']


class EResearchContactIdentifierAdmin(admin.ModelAdmin):
    list_display = ('identifier', 'contact',
                    'system', 'parent_erb_contact_id', 'provision_details')
    list_filter = ['provision_details__status', 'system', 'contact']


class DelegatesInline(admin.TabularInline):
    model = models.CramsERBUserRoles.delegates.through
    verbose_name = "Delegate"
    verbose_name_plural = "Delegates"


class ErbSystemAdminInline(admin.TabularInline):
    model = models.CramsERBUserRoles.admin_erb_systems.through
    verbose_name = "Admin ERB System"
    verbose_name_plural = "Admin ERB Systems"


class ErbSystemApproverInline(admin.TabularInline):
    model = models.CramsERBUserRoles.approver_erb_systems.through
    verbose_name = "Approver ERB System"
    verbose_name_plural = "Approver ERB Systems"


class ProviderInline(admin.TabularInline):
    model = models.CramsERBUserRoles.providers.through
    verbose_name = "Provider"
    verbose_name_plural = "Providers"


class CramsERBUserRolesAdmin(admin.ModelAdmin):
    list_display = ('contact', 'role_erb', 'is_erb_admin')
    inlines = [
        DelegatesInline,
        ErbSystemAdminInline,
        ErbSystemApproverInline,
        ProviderInline
    ]
    exclude = ('delegates', 'admin_erb_systems',
               'approver_erb_systems', 'providers')


class FacultyAdmin(admin.ModelAdmin):
    """
    class FacultyAdmin
    """
    list_display = ('name', 'organisation', 'faculty_code')
    list_filter = ['organisation']


class SupportEmailContactAdmin(admin.ModelAdmin):
    """
    class SupportEmailContact
    """
    list_display = ('description', 'email',
                    'e_research_body', 'e_research_system')
    list_filter = ['e_research_body', 'e_research_system']


class OrganisationContactAdmin(admin.ModelAdmin):
    """
    class UserRoleAdmin
    """
    list_display = ('contact', 'organisation', 'faculty', 'department')
    list_filter = ['contact', 'faculty', 'organisation']


class ContactFacultyAdmin(admin.ModelAdmin):
    """
    class ContactFacultyAdmin
    """
    list_display = ('contact', 'faculty', 'archived_ts', 'type', 'school_name')
    list_filter = ['faculty', 'contact', 'type', 'contact__email']


admin.site.register(models.ContactDetail, ContactDetailAdmin)
admin.site.register(models.Contact, ContactAdmin)
admin.site.register(models.ContactRole,  ContactRoleAdmin)
admin.site.register(
    models.EResearchContactIdentifier, EResearchContactIdentifierAdmin)
admin.site.register(models.Organisation, OrganisationAdmin)
# admin.site.register(models.OrganisationContact, OrganisationContactAdmin)
admin.site.register(models.CramsERBUserRoles, CramsERBUserRolesAdmin)
admin.site.register(models.Faculty, FacultyAdmin)
# admin.site.register(models.SupportEmailContact, SupportEmailContactAdmin)
# admin.site.register(models.ContactFaculty, ContactFacultyAdmin)



# coding=utf-8
"""
contact Serializers
"""
import logging

from django.contrib import auth
from django.db.models import Q
from rest_framework import serializers, exceptions
from rest_framework.validators import UniqueValidator

from crams.serializers import model_serializers
from crams_contact import models as contact_models
from crams_contact.config import contact_config
from crams_contact.serializers import lookup_serializers

LOG = logging.getLogger(__name__)
User = auth.get_user_model()


class RoleFreeContactDetailsLookupSerializer(model_serializers.ModelLookupSerializer):
    id = serializers.IntegerField(required=False)
    
    title = serializers.CharField(required=False, allow_null=True)

    given_name = serializers.CharField(required=False, allow_null=True)

    surname = serializers.CharField(required=False, allow_null=True)

    orcid = serializers.CharField(required=False, allow_null=True)

    scopusid = serializers.CharField(required=False, allow_null=True)

    email = serializers.EmailField(validators=[])

    phone = serializers.SerializerMethodField(required=False, allow_null=True)

    organisation = lookup_serializers.OrganisationLookupSerializer(
        many=False, required=False, allow_null=True)

    contact_details = serializers.SerializerMethodField()

    class Meta(object):
        model = contact_models.Contact
        fields = ('id', 'title', 'given_name', 'surname', 'email', 'phone',
                  'organisation', 'orcid', 'scopusid', 'contact_details', 'latest_contact')
        read_only_fields = ('title', 'given_name', 'surname', 'orcid',
                            'scopusid', 'phone', 'organisation',
                            'contact_details', 'id', 'latest_contact')
        pk_fields = ['email']

    def create(self, validated_data):
        raise exceptions.ParseError('Create not allowed ')

    def update(self, instance, validated_data):
        raise exceptions.ParseError('Update not allowed ')

    def __delete__(self, instance):
        raise exceptions.ParseError('Delete not allowed ')

    @classmethod
    def fetch_first_contact_obj(cls, validated_data):
        return cls.fetch_contacts(validated_data).first()

    @classmethod
    def fetch_contacts(cls, validated_data):
        email = validated_data.get('email')
        if email:
            qs_filter = Q(email__iexact=email) | Q(
                contact_details__email__iexact=email)
        else:
            pk = validated_data.get('id')
            qs_filter = Q(pk=pk)

        if not qs_filter:
            return contact_models.Contact.objects.none()
        return contact_models.Contact.objects.filter(qs_filter).distinct()

    @classmethod
    def search_contact_get_queryset(cls, pk=None, email=None):
        search_dict = dict()
        if pk:
            search_dict['id'] = pk
        if email:
            search_dict['email'] = email
        if search_dict:
            return cls.fetch_contacts(search_dict)
        return None

    @classmethod
    def get_details_queryset(cls, contact):
        qs = contact.contact_details
        return qs.filter(restricted=False).order_by('type')

    @classmethod
    def get_email(cls, contact):
        if not contact.email:
            for detail in cls.get_details_queryset(contact):
                if detail.email:
                    return detail.email
        return contact.email

    @classmethod
    def get_organisation(cls, contact):
        if contact.organisation:
            return contact.organisation
        else:
            return None

    def get_phone(self, contact):
        def get_first_available_phone_number():
            if not contact.phone:
                for detail in self.get_details_queryset(contact):
                    phone = detail.phone
                    if phone:
                        return phone
            return contact.phone

        return get_first_available_phone_number()

    def get_contact_details(self, contact):
        ret_list = list()
        for detail in self.get_details_queryset(contact):
            detail_dict = dict()
            detail_dict['type'] = detail.get_type_display()
            detail_dict['phone'] = detail.phone
            detail_dict['email'] = detail.email
            ret_list.append(detail_dict)
        return ret_list

    @classmethod
    def get_related_emails(cls, contact_obj):
        if contact_obj:
            email_set = set()
            email_set.add(cls.format_email(contact_obj.email))
            for detail in contact_obj.contact_details.all():
                if detail.email:
                    email_set.add(cls.format_email(detail.email))
            return list(email_set)
        return list()

    @classmethod
    def format_email(cls, email_in):
        if email_in:
            return email_in.strip().lower()
        return None

    def validate(self, attrs):
        print('RoleFreeContactDetailsLookupSerializer', attrs)
        super().validate(attrs)
        self.verify_field_value('email', attrs, ignore_case=True)
        print('returning ', self.instance)
        return self.instance or attrs


class RoleFreeContactSerializer(RoleFreeContactDetailsLookupSerializer):
    email = serializers.EmailField(
        required=True, validators=[
            UniqueValidator(queryset=contact_models.Contact.objects.all())])

    organisation = lookup_serializers.OrganisationLookupSerializer(
        many=False, required=False, allow_null=True)

    class Meta(object):
        model = contact_models.Contact
        fields = ('id', 'title', 'given_name', 'surname', 'email', 'phone',
                  'organisation', 'orcid', 'scopusid', 'contact_details',
                  'latest_contact', 'contact_ids')
        read_only_fields = ()
        pk_fields = ['email']

    @classmethod
    def add_related_entities(cls, contact, validated_data):
        related_tables_map = {
            'contact_details': contact_models.ContactDetail
        }

        for key, model in related_tables_map.items():
            data = validated_data.get(key)
            if data:
                for instance_data in data:
                    instance = model(**instance_data)
                    instance.parent_contact = contact
                    instance.save()

    @classmethod
    def validate_email(cls, email):
        email = cls.format_email(email)
        if not email:
            msg = 'Email required, cannot resolve to None'
            raise exceptions.ValidationError(msg)
        return email

    @classmethod
    def validate_contact_details(cls, contact_details):
        if contact_details:
            for c_d in contact_details:
                c_d['email'] = cls.format_email(c_d.get('email'))
        return contact_details

    def validate(self, data):
        # the following fields are serializer method fields
        data['phone'] = self.initial_data.get('phone')

        contact_details = self.initial_data.get('contact_details')
        if contact_details:
            self.validate_contact_details(contact_details)
            data['contact_details'] = contact_details

        latest_contact = data.get('latest_contact')
        if latest_contact:
            msg = 'historical Contact, edit current contact instead'
            raise exceptions.ValidationError(msg)

        return data

    @classmethod
    def setup_default_contact_detail(cls, parent_contact):
        contact_models.ContactDetail(
            type=contact_models.ContactDetail.BUSINESS,
            parent_contact=parent_contact,
            phone=parent_contact.phone,
            email=parent_contact.email).save()

    @classmethod
    def create(cls, validated_data):
        try:
            contact = cls.fetch_first_contact_obj(validated_data)
            if contact:
                email = validated_data.get('email')
                msg = 'Contact exists for email: ' + email
                raise exceptions.ParseError(msg)
        except contact_models.Contact.DoesNotExist:
            pass

        related_tables_data = dict()
        contact_details = validated_data.pop('contact_details', None)

        contact = contact_models.Contact.objects.create(**validated_data)
        if not contact_details:
            cls.setup_default_contact_detail(contact)
        else:
            related_tables_data['contact_details'] = contact_details
            cls.add_related_entities(contact, related_tables_data)

        # create/update default ERB Body Contact Ids for this contact
        contact_config.update_erb_contact_ids_for_key(contact)

        return contact

    @classmethod
    def update(cls, instance, validated_data):
        if 'title' in validated_data:
            instance.title = validated_data.get('title')
        if 'surname' in validated_data:
            instance.surname = validated_data.get('surname')
        if 'given_name' in validated_data:
            instance.given_name = validated_data.get('given_name')
        if 'phone' in validated_data:
            instance.phone = validated_data.get('phone')
        if 'orcid' in validated_data:
            instance.orcid = validated_data.get('orcid')
        if 'scopusid' in validated_data:
            instance.scopusid = validated_data.get('scopusid')
        if 'email' in validated_data:
            instance.email = validated_data.get('email')
        if 'organisation' in validated_data:
            instance.organisation = validated_data.get('organisation')

        instance.save()

        contact_details = validated_data.get('contact_details')
        if contact_details:
            for detail in contact_details:
                email = detail.get('email')
                phone = detail.get('phone')
                type_str = detail.get('type')
                if not type_str:
                    msg = 'Contact Detail Type required'
                    raise exceptions.ValidationError(msg)

                contact_type = contact_models.ContactDetail.get_choice_value(type_str)
                if not contact_type:
                    msg = 'Contact Detail Type {} not valid'.format(type_str)
                    raise exceptions.ValidationError(msg)

                try:
                    detail_obj = \
                        instance.contact_details.get(type=contact_type)
                    detail_obj.email = email
                    detail_obj.phone = phone
                    detail_obj.save()
                except contact_models.ContactDetail.DoesNotExist:
                    contact_models.ContactDetail(
                        type=contact_type, parent_contact=instance,
                        phone=phone, email=email).save()
                except contact_models.ContactDetail.MultipleObjectsReturned:
                    msg = \
                        'Data error, multiple details exists for contact type'
                    raise exceptions.ValidationError(msg.format(type_str))

        if not instance.contact_details.exists():
            cls.setup_default_contact_detail(instance)

        # create/update default ERB Body Contact Ids for this contact
        contact_config.update_erb_contact_ids_for_key(instance)

        return instance



from crams.models import EResearchBody
from crams.test_utils import CommonBaseTestCase
from crams.utils import crams_utils
from django.contrib.auth import get_user_model
from mixer.backend.django import mixer

from crams_contact.models import Contact
from crams_contact.models import CramsERBUserRoles
from crams_contact.models import ContactRole
from crams_contact.serializers.contact_erb_roles import ContactErbRoleSerializer
User = get_user_model()


class UnitTestCase(CommonBaseTestCase):

    def generate_user_and_contact(self, user=None, org=None):
        if user is None:
            user = self.generate_new_user()
        contact = mixer.blend(Contact, 
                              given_name=user.username,
                              surname=user.username,
                              email=user.email,
                              organisation=org)

        return user, contact
    
    def generate_erb_admin_user_and_token(self, erb=None, user=None, org=None):
        # if user is None:
        #     user = self.generate_new_user()
        # contact = mixer.blend(Contact, email=user.email)
        user, contact = self.generate_user_and_contact(user=user, org=org)

        erb_prefix = crams_utils.get_random_string(8)
        if not erb:
            erb = mixer.blend(EResearchBody, name=erb_prefix + '_erb', email=erb_prefix + '@crams.org')
        mixer.blend(CramsERBUserRoles, role_erb=erb, is_erb_admin=True, contact=contact)
        crams_token = ContactErbRoleSerializer.setup_crams_token_and_roles(user_obj=user)
        return user, contact, crams_token

    def generate_contact_role(self, name, erb=None, project_leader=False, read_only=False, support_notification=False):
        return mixer.blend(ContactRole, 
                           name=name, 
                           e_research_body=erb, 
                           project_leader=project_leader,
                           read_only=read_only,
                           support_notification=support_notification)
    


from rest_framework import mixins
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from crams.utils import django_utils
from crams import permissions
from crams_contact.serializers.organisation_serializers import OrganisationSerializer
from crams_contact.serializers.organisation_serializers import FacultySerializer
from crams_contact.serializers.organisation_serializers import DepartmentSerializer
from crams_contact.serializers.organisation_serializers import DepartmentListSerializer
from crams_contact.serializers.organisation_serializers import DepartmentParentSerializer
from crams_contact.models import Department
from crams_contact.models import Faculty
from crams_contact.models import Organisation


class OrganisationViewSet(django_utils.CramsModelViewSet):
    """
    class OrganisationViewSet
    """
    permission_classes = [permissions.IsCramsAuthenticated,
                          permissions.IsLookupAdmin]
    queryset = Organisation.objects.all()
    serializer_class = OrganisationSerializer


class FacultyViewSet(django_utils.CramsModelViewSet):
    """
    class FacultyViewSet
    """
    permission_classes = [permissions.IsCramsAuthenticated,
                          permissions.IsLookupAdmin]
    queryset = Faculty.objects.all().order_by('name')
    serializer_class = FacultySerializer

    @action(detail=False, url_path='organisation/(?P<org_id>\d+)')
    def get_org_faculty(self, request, org_id):
        ret_val = dict()
        if org_id:
            queryset = self.queryset.filter(organisation__id=org_id)
            ret_val = self.serializer_class(queryset, many=True).data
        return Response(ret_val)


class DepartmentViewSet(django_utils.CramsModelViewSet):
    """
    class DepartmentViewSet
    """
    permission_classes = [permissions.IsCramsAuthenticated,
                          permissions.IsLookupAdmin]
    queryset = Department.objects.all().order_by('name')
    serializer_class = DepartmentSerializer

    @action(detail=False, url_path='faculty/(?P<faculty_id>\d+)')
    def get_faculty_department(self, request, faculty_id):
        ret_val = dict()
        if faculty_id:
            queryset = self.queryset.filter(faculty__id=faculty_id)
            ret_val = self.serializer_class(queryset, many=True).data
        return Response(ret_val)

# TOOD: fix base
# class FacultyListViewSet(mixins.RetrieveModelMixin,
#                          base.BaseCramsReportPrimaryKeyViewSet):
class FacultyListViewSet(mixins.RetrieveModelMixin):
    """
    class FacultyListViewSet
    """
    permission_classes = [permissions.IsCramsAuthenticated]
    queryset = Faculty.objects.none()

    @classmethod
    def build_org_data_dict(cls, org, faculty_list):
        org_data_dict = dict()
        org_data_dict['id'] = org.id
        org_data_dict['short_name'] = org.short_name
        org_data_dict['name'] = org.name
        org_data_dict['faculties'] = FacultySerializer(faculty_list,
                                                       many=True).data
        return org_data_dict

    def build_pk_view_response(self, http_request, faculty_qs):
        faculty_list = list()
        if self.pk_param:
            try:
                org = Organisation.objects.get(pk=self.pk_param)
            except Organisation.DoesNotExist:
                msg = 'Organisation with pk {} does not exists'
                raise exceptions.ValidationError(msg.format(self.pk_param))

            for f in faculty_qs:
                if f.organisation == org:
                    faculty_list.append(f)
            return Response(self.build_org_data_dict(org, faculty_list))
        return Response(dict())

    def build_view_response(self, http_request, faculty_qs):
        if hasattr(self, 'pk_param'):
            return self.build_pk_view_response(http_request, faculty_qs)

        org_faculty_dict = dict()
        for faculty in faculty_qs:
            f_list = org_faculty_dict.get(faculty.organisation)
            if not f_list:
                f_list = list()
                org_faculty_dict[faculty.organisation] = f_list
            f_list.append(faculty)

        ret_list = list()
        for org, faculty_list in org_faculty_dict.items():
            ret_list.append(self.build_org_data_dict(org, faculty_list))

        return Response(ret_list)

    def build_admin_queryset(self, qs, erbs_list):
        return Faculty.objects.all()

    def build_faculty_queryset(self, qs, contact):
        return user_utils.fetch_contact_faculties_qs(contact)

    def build_department_queryset(self, qs, contact):
        return self.queryset

    def build_organisation_queryset(self, qs, contact):
        return user_utils.fetch_contact_faculties_qs(contact)

    def build_null_role_queryset(self, qs, contact):
        return self.queryset


class DepartmentListViewSet(viewsets.GenericViewSet,
                            mixins.RetrieveModelMixin):
    """
    class DepartmentListViewSet
    """
    permission_classes = [permissions.IsCramsAuthenticated,
                          permissions.IsLookupAdmin]
    queryset = Faculty.objects.all()
    serializer_class = DepartmentListSerializer


class DepartmentParentViewSet(viewsets.GenericViewSet,
                              mixins.RetrieveModelMixin):
    """
    class DepartmentParentViewSet
    """
    permission_classes = [permissions.IsCramsAuthenticated,
                          permissions.IsLookupAdmin]
    queryset = Department.objects.all()
    serializer_class = DepartmentParentSerializer


# Generated by Django 3.1.1 on 2021-03-09 08:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crams_log', '0002_userlog'),
        ('crams_contact', '0002_load_contact_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContactLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('crams_contact_db_id', models.IntegerField(blank=True, null=True)),
                ('log_parent', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='contactlog', to='crams_log.cramslog')),
            ],
            options={
                'unique_together': {('crams_contact_db_id', 'log_parent')},
            },
        ),
    ]



from django.utils.translation import ugettext_lazy as _
from rest_framework import authentication, exceptions
from crams import models


class CramsTokenAuthentication(authentication.TokenAuthentication):
    message = _('User does not hold valid CramsToken.')

    def authenticate(self, request):
        auth_tuple = super().authenticate(request)
        if auth_tuple:
            try:
                user, token = auth_tuple
                crams_token = models.CramsToken.objects.get(user=user)
                if crams_token.is_expired():
                    raise exceptions.AuthenticationFailed(self.message)
            except Exception as e:
                raise exceptions.AuthenticationFailed(_('{}'.format(e)))
        return auth_tuple



from rest_framework import serializers

from crams_contact.models import Department, Faculty, Organisation
from crams.serializers import utilitySerializers
from crams.serializers.model_serializers import ReadOnlyModelSerializer


class ReadOnlyOrganisationSerializer(ReadOnlyModelSerializer):
    """class OrganisationSerializer."""

    class Meta(object):
        """meta Class."""
        model = Organisation
        fields = ('id', 'name', 'short_name')


class OrganisationSerializer(serializers.ModelSerializer, utilitySerializers.UpdatableSerializer):
    """class OrganisationSerializer."""

    class Meta(object):
        """meta Class."""

        model = Organisation
        fields = ['id', 'name', 'short_name', 'ands_url', 'notification_email']

    def create(self, validated_data):
        """create.

        :param validated_data:
        :return:
        """
        return Organisation.objects.create(**validated_data)


class FacultySerializer(serializers.ModelSerializer, utilitySerializers.UpdatableSerializer):
    """class FacultySerializer."""

    class Meta(object):
        """meta Class."""
        model = Faculty
        fields = ['id', 'faculty_code', 'name', 'organisation']

    def create(self, validated_data):
        """create.

        :param validated_data:
        :return:
        """
        return Faculty.objects.create(**validated_data)


class DepartmentSerializer(serializers.ModelSerializer,
                           utilitySerializers.UpdatableSerializer):
    """class DepartmentSerializer"""

    class Meta(object):
        """meta Class."""
        model = Department
        fields = ['id', 'department_code', 'name', 'faculty']

    def create(self, validated_data):
        """create.

        :param validated_data:
        :return:
        """
        return Department.objects.create(**validated_data)


class DepartmentListSerializer(serializers.ModelSerializer):
    """
    class DepartmentListSerializer
    Lists all department belonging to an faculty
    """
    departments = serializers.SerializerMethodField(
        method_name='get_department_list')

    class Meta(object):
        """meta Class"""
        model = Faculty
        fields = ['id', 'name', 'departments']

    # ordered by department name
    @classmethod
    def get_department_list(cls, obj):
        query_set = Department.objects.filter(
            faculty_id=obj.id).order_by('name')
        return DepartmentSerializer(query_set, many=True).data


class DepartmentParentSerializer(serializers.ModelSerializer):
    """
    class DepartmentParentSerializer
    Combines Organisation, Faculty and Department into 1
    """
    department = DepartmentSerializer
    faculty = serializers.SerializerMethodField(method_name='filter_faculty')
    organisation = \
        serializers.SerializerMethodField(method_name='filter_organisation')

    class Meta(object):
        """meta Class"""
        model = Department
        fields = ['id', 'department_code', 'name', 'faculty', 'organisation']

    @classmethod
    def filter_faculty(cls, dept_obj):
        serializer = FacultySerializer(dept_obj.faculty)

        return serializer.data

    @classmethod
    def filter_organisation(cls, dept_obj):
        serializer = OrganisationSerializer(dept_obj.faculty.organisation)

        return serializer.data



# coding=utf-8
"""
 auth.py
"""
from crams.models import UserEvents
from crams_log.log_events import login
from django.contrib.auth import authenticate
from rest_framework.authtoken import views
from rest_framework.response import Response

from crams_contact.serializers.contact_erb_roles import ContactErbRoleSerializer


class CramsBasicLoginAuthToken(views.ObtainAuthToken):
    """

    """

    def post(self, request):
        """
        post method
        :param request:
        :return:
        """
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        username = serializer.validated_data['username']
        password = serializer.validated_data['password']
        user = authenticate(username=username, password=password)
        if user is not None:
            # msg = 'User logged in with valid User/Password'
            # login.log_user_login(request, user, msg)
            # UserEvents.objects.create(
            #     created_by=user,
            #     event_message=msg
            # )

            crams_token = ContactErbRoleSerializer.setup_crams_token_and_roles(user)

            response = Response({'token': crams_token.key})
            response['token'] = crams_token.key
            response['roles'] = crams_token.ks_roles
            return response

        return Response('Login fail')

# obtain_auth_token = CramsBasicLoginAuthToken.as_view()



# coding=utf-8
"""

"""
import copy
from rest_framework import serializers, exceptions

from crams.serializers import erb_serializers
from crams_contact.utils import contact_utils
from crams_contact import models as contact_models
from crams_contact.serializers import util_sz
from crams_contact.serializers.role_free_contact_serializer import RoleFreeContactSerializer


class ContactIDSerializer(erb_serializers.BaseIdentifierSerializer):
    class Meta(object):
        """meta Class."""

        model = contact_models.EResearchContactIdentifier
        fields = ('identifier', 'system')

    @classmethod
    def build_contact_id_list_json(
            cls, contact_id_obj_list, json_id_key='contact_ids'):
        def build_json(contact_obj, pk_list):
            sz = cls(pk_list, many=True)
            return {
                'id': contact_obj.id,
                'given_name': contact_obj.given_name,
                'surname': contact_obj.surname,
                'email': contact_obj.email,
                json_id_key: sz.data
            }

        contact_dict = dict()
        for cid in contact_id_obj_list:
            id_list = contact_dict.get(cid.contact)
            if not id_list:
                id_list = list()
                contact_dict[cid.contact] = id_list
            id_list.append(cid)

        ret_list = list()
        for contact, id_list in contact_dict.items():
            ret_list.append(build_json(contact, id_list))
        return ret_list

    @classmethod
    def update_contact_data_list(cls, id_key, contact_data_list):
        contact_id_obj_list = list()
        for contact_data in contact_data_list:
            for sz in contact_data.get(id_key, list()):
                contact_id_obj_list.append(sz.save())

        return contact_id_obj_list

    @classmethod
    def validate_contact_identifiers_in_contact_list(cls, contact_data_list, current_user, id_key='contact_ids'):
        def build_contact_error(contact_data_to_copy, err_message, errors=None):
            err_dict = copy.copy(contact_data_to_copy)
            err_dict.pop('contact_obj', None)
            err_dict['error'] = True
            err_dict['error_message'] = err_message
            if errors:
                err_dict[id_key] = errors
            return err_dict
        role_fn = contact_utils.fetch_erb_userroles_with_provision_privileges
        user_erb_roles = role_fn(current_user)
        err_list = list()
        for contact_data in contact_data_list:
            contact_ids = contact_data.pop(id_key, list())
            contact = RoleFreeContactSerializer.fetch_first_contact_obj(contact_data)
            if not contact:
                msg = 'Contact not found'
                err_list.append(build_contact_error(contact_data, msg))
                continue

            contact_data['contact_obj'] = contact
            id_err_list = list()
            contact_id_sz_list = list()
            contact_data[id_key] = contact_id_sz_list
            context = {
                'contact': contact,
                'user_erb_roles': user_erb_roles,
                'current_user': current_user
            }
            for sid in contact_ids:
                sz = cls(data=sid, context=context)
                sz.is_valid(raise_exception=False)
                if sz.errors:
                    sid['error_message'] = sz.errors
                    id_err_list.append(sid)
                    continue
                contact_id_sz_list.append(sz)

            if id_err_list:
                msg = 'Error processing Contact Ids'
                err_list.append(build_contact_error(
                    contact_data, msg, id_err_list))

        if err_list:
            raise exceptions.ValidationError(err_list)

        return contact_data_list

    def validate_contact(self, validated_data):
        contact = validated_data.get('contact')
        if not contact or not isinstance(contact, contact_models.Contact):
            msg = 'Contact object must be passed in context'
            raise exceptions.ValidationError(msg)

        self.instance = self.fetch_identifier_unique_for_system(
            validated_data, model=contact_models.EResearchContactIdentifier)

    def validate(self, data):
        validated_data = dict()
        validated_data['system'] = data.get('system')
        validated_data['parent_erb_contact_id'] = None

        contact = self.context.get('contact')
        if contact and isinstance(contact, contact_models.Contact):
            validated_data['contact'] = contact
            self.validate_contact(validated_data)

        validated_data['identifier'] = data.get('identifier')
        return validated_data

    def create(self, validated_data):
        self.validate_contact(validated_data)
        if self.instance:
            return self.update(self.instance, validated_data)
        return self.Meta.model.objects.create(**validated_data)

    def update(self, instance, validated_data):
        new_instance = self.Meta.model.objects.create(**validated_data)

        # Archive previous value for identifier
        instance.parent_erb_contact_id = new_instance
        instance.save()

        # Update archived objects parent_id with latest Identifier object
        qs = contact_models.EResearchContactIdentifier.objects.filter(
            parent_erb_contact_id=instance)
        if qs.exists():
            for cid in qs.all():
                cid.parent_erb_contact_id = new_instance
                cid.save()

        return new_instance


class ContactSystemIdentifierSz(util_sz.EmptySerializer):
    class ContactIdExImSerializer(util_sz.EmptySerializer):
        class SystemId(util_sz.EmptySerializer):
            key = serializers.CharField()
            e_research_body = serializers.CharField()

        system = SystemId()
        identifier = serializers.CharField()

    system_identifiers = serializers.SerializerMethodField()
    first_name = serializers.CharField(source='given_name', required=False)
    last_name = serializers.CharField(source='surname', required=False)
    email = serializers.EmailField()
    id = serializers.IntegerField(required=False)

    def get_system_identifiers(self, contact_obj):
        id_list = contact_obj.system_identifiers
        valid_erb_list = self.context.get('valid_erb_list', None)
        if not valid_erb_list:
            return self.ContactIdExImSerializer(id_list, many=True).data

        ret_list = list()
        for erb_contact_id_obj in id_list:
            if erb_contact_id_obj.system.e_research_body in valid_erb_list:
                ret_list.append(self.ContactIdExImSerializer(erb_contact_id_obj).data)
        return ret_list

    class Meta(object):
        fields = ('id', 'first_name', 'last_name',
                  'email', 'system_identifiers')
        read_only_fields = ('first_name', 'last_name', 'id')



# coding=utf-8
"""

"""

from crams.extension import config_init
from crams.utils import rest_utils
from crams import models as crams_models
from crams_contact import models as contact_models
from crams.utils.role import AbstractCramsRoleUtils


def system_identifiers(contact_obj):
    return contact_models.EResearchContactIdentifier.objects.filter(
        contact=contact_obj, parent_erb_contact_id__isnull=True)


def fetch_contact_for_user(user_obj):
    if isinstance(user_obj, crams_models.User):
        qs = contact_models.Contact.objects.filter(
            email__iexact=user_obj.email, latest_contact__isnull=True)
        if qs.exists():
            return qs.first()
    return None


def fetch_user_contact_if_exists(crams_user):
    return fetch_contact_for_user(crams_user)


def user_details_json(user_obj, null_default='System'):
    if user_obj:
        contact = fetch_user_contact_if_exists(user_obj)
        if contact:
            return {
                'email': contact.email,
                'first_name': contact.given_name,
                'last_name': contact.surname
            }
        return {
            'email': user_obj.email,
            'first_name': user_obj.first_name,
            'last_name': user_obj.last_name
        }
    return {
        'email': null_default,
        'first_name': null_default,
        'last_name': null_default
    }


def fetch_related_user_erb_system_list(serializer_self):
    user_obj, _ = rest_utils.get_current_user_from_context(serializer_self)
    if user_obj:
        return AbstractCramsRoleUtils.get_authorised_e_research_system_list(user_obj)
    return list()


def fetch_related_user_erb_list(serializer_self):
    ret_set = set()
    for erbs in fetch_related_user_erb_system_list(serializer_self):
        ret_set.add(erbs.e_research_body)
    return list(ret_set)


def fetch_erb_userroles_with_provision_privileges(
        crams_user, e_research_body_obj=None):
    """

    :param crams_user:
    :param e_research_body_obj:
    :return:
    """
    ret_set = set()
    if isinstance(crams_user, crams_models.User):
        qs = contact_models.CramsERBUserRoles.objects.all()
        contact = fetch_user_contact_if_exists(crams_user)
        if contact:
            qs = qs.filter(contact=contact)
        if e_research_body_obj:
            qs = qs.filter(role_erb=e_research_body_obj)

        for erb_user_role in qs.all():
            erb = erb_user_role.role_erb
            if erb_user_role.providers.exists():
                ret_set.add(erb_user_role)
                continue

            if erb_user_role.is_erb_admin:
                if erb not in config_init.ERB_PROVIDER_DISALLOW_LIST:
                    ret_set.add(erb_user_role)
                    continue

            for erbs in erb_user_role.admin_erb_systems.all():
                if erbs not in config_init.ERBS_PROVIDER_DISALLOW_LIST:
                    ret_set.add(erb_user_role)
                    break

    return list(ret_set)


def fetch_contact_delegates_for_erb_systems(contact, erb_systems):
    erb_set = set()
    for erbs in erb_systems:
        erb_set.add(erbs.e_research_body)
    role_qs = contact.user_roles.filter(role_erb__in=erb_set,
                                        delegates__isnull=False)
    ret_set = set()
    if role_qs.exists():
        for erb_role in role_qs.all():
            ret_set = ret_set | set(erb_role.delegates.all())
    return list(ret_set)



from django.contrib import admin
from crams_manager import models
from crams_contact import models as contact_models


class ServiceManagerAdmin(admin.ModelAdmin):
    list_display = ('contact', 'active')
    model = models.ServiceManager
    model._meta.verbose_name = 'ServiceManager (MeRC Management)'
    model._meta.verbose_name_plural = 'ServiceManagers (MeRC Management)'


class DepartmentManagerAdmin(admin.ModelAdmin):
    list_display = ('contact', 'department', 'active')
    list_filter = ['department', 'department__faculty',
                   'department__faculty__organisation']

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "department":
            kwargs["queryset"] = contact_models.Department.objects.all(). \
                order_by('faculty__organisation', 'faculty', 'name')
        return super().formfield_for_foreignkey(db_field, request, **kwargs)


class FacultyManagerAdmin(admin.ModelAdmin):
    list_display = ('contact', 'faculty', 'active')
    list_filter = ['faculty', 'faculty__organisation']

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "faculty":
            kwargs["queryset"] = contact_models.Faculty.objects.all(). \
                order_by('organisation', 'name')
        return super().formfield_for_foreignkey(db_field, request, **kwargs)


class OrganisationManagerAdmin(admin.ModelAdmin):
    list_display = ('contact', 'organisation', 'active')


admin.site.register(models.ServiceManager, ServiceManagerAdmin)
admin.site.register(models.DepartmentManager, DepartmentManagerAdmin)
admin.site.register(models.FacultyManager, FacultyManagerAdmin)
admin.site.register(models.OrganisationManager, OrganisationManagerAdmin)



"""crams_contact URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from crams_contact.auth import rapidconnect_auth
from crams_contact.auth.auth_api import CramsBasicLoginAuthToken
from crams_contact.auth.user_roles_api import CurrentUserRolesView
from crams_contact.views.contact import ContactViewSet, AdminContactViewSet
from crams_contact.views.contact_role import ContactRoleViewSet
from crams_contact.views.organisation import DepartmentViewSet
from crams_contact.views.organisation import FacultyViewSet
from crams_contact.views.organisation import OrganisationViewSet
from django.conf.urls import url
from django.urls import path, include
from rest_framework import routers

router = routers.SimpleRouter()
router.register(r'contact_role', ContactRoleViewSet)
router.register(r'department', DepartmentViewSet)
router.register(r'faculty', FacultyViewSet)
router.register(r'organisation', OrganisationViewSet)
router.register(r'searchcontact', ContactViewSet)
router.register(r'admin_contact', AdminContactViewSet)

urlpatterns = [
    path('', include(router.urls)),
    url(r'api-token-auth', CramsBasicLoginAuthToken.as_view(), name='api_token_auth'),
    url(r'user_roles', CurrentUserRolesView.as_view(), name='user_roles'),
    # rapid connect authentication related
    url(r'redirect_to_rapid_conn', rapidconnect_auth.redirect_to_rapid_conn),
    url(r'rapid_conn_auth', rapidconnect_auth.rapid_conn_auth_view),
    # path('admin/', admin.site.urls),
]



# coding=utf-8
"""
 rapid connect authentication related code
"""
import jwt
from django.http import HttpResponseRedirect
from django.views.decorators.csrf import csrf_exempt
from rest_framework import exceptions as rest_exceptions
from rest_framework.decorators import api_view
from django.conf import settings

# from crams.common.utils import log_process
from crams.utils.lang_utils import strip_lower
from crams.models import User
# from crams.models import UserEvents
from crams_contact.serializers.contact_erb_roles import ContactErbRoleSerializer


@api_view(['GET'])
def redirect_to_rapid_conn(request):
    """
    redirect to AAF Rapid Connect
    :param request:
    :return:
    """
    response = HttpResponseRedirect(settings.AAF_RAPID_CONN_URL)
    # save the origin address of where the request came from
    # the origin address is used to redirect back after auth is successful
    request.session["HTTP_REFERER"] = request.META['HTTP_REFERER']
    request.session.modified = True
    return response


@csrf_exempt
def rapid_conn_auth_view(request):
    """
    AAF Rapid Connect auth service POST
    :param request: request should contain a jwt
    :return:
    """
    # decode a jwt using the secret key
    try:
        # fetch the server and port number
        try:
            host = request.headers['Host']
            server = host.split(':')[0]
            port = host.split(':')[1]
        except:
            server = host
            port = None
        
        # set the header HTTPS if port is 443
        if port == '443':
            header = 'https://'
        else:
            header = 'http://'

        verified_jwt = jwt.decode(
            request.POST['assertion'], settings.AAF_SECRET,
            audience=header + server,
            options={'verify_iat': False})
    except:
        raise rest_exceptions.AuthenticationFailed(
            'Invalid jwt from rapid connect, contact Support')

    # get user details from jwt
    email = verified_jwt['https://aaf.edu.au/attributes']['mail']
    email = strip_lower(email)

    # user name attributes
    display_name = verified_jwt['https://aaf.edu.au/attributes']['displayname']
    first_name = verified_jwt['https://aaf.edu.au/attributes']['givenname']
    last_name = verified_jwt['https://aaf.edu.au/attributes']['surname']

    # verify user on crams
    try:
        user = User.objects.get(email=email,
                                first_name=first_name,
                                last_name=last_name)

    # return multiple users with the same email
    except User.MultipleObjectsReturned:
        raise rest_exceptions.AuthenticationFailed(
            'Multiple UserIds exist for User, contact Support')

    # User does not exist in CRAMS
    except User.DoesNotExist:
        user, created = User.objects.get_or_create(email=email, username=email)

    #log login
    # msg = 'User logged in with valid AAF credentials'
    # log_process.log_user_login(request, user, msg)
    # events = UserEvents(
    #     created_by=user,
    #     event_message=msg
    # )
    # events.save()

    # Generate CramsToken
    crams_token = ContactErbRoleSerializer.setup_crams_token_and_roles(user)
    query_string = "?username=%s&rest_token=%s" % (user.username,
                                                   crams_token.key)
    
    response = HttpResponseRedirect(request.session.get("HTTP_REFERER") +
                                    '/#/ks-login' +
                                    query_string)
    response['token'] = crams_token.key
    response['roles'] = crams_token.ks_roles

    return response



import re

from django.core.exceptions import ValidationError


def validate_orcid(value):
    pattern = re.compile("http://orcid.org/\w{4}-\w{4}-\w{4}-\w{4}")
    https_pattern = re.compile("https://orcid.org/\w{4}-\w{4}-\w{4}-\w{4}")
    if not pattern.match(value) and not https_pattern.match(value):
        err_msg = 'required format: http://orcid.org/XXXX-XXXX-XXXX-XXXX, ' \
                  'where X is [0-9,a-z,A-Z]'
        raise ValidationError(err_msg)
    else:
        return



# coding=utf-8
"""
Common Utils type Serializers
"""
# import ast
import pprint

from rest_framework import serializers, exceptions

from crams.utils import rest_utils, action_state
from crams_contact.utils import contact_utils
from crams_contact import models as contact_models
from crams.utils.role import AbstractCramsRoleUtils
from crams.extension.crams_aspects import CramsAspect, CRAMS_ASPECT_DICT


class EmptySerializer(serializers.Serializer):
    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass


class CramsAspectSerializer:
    @classmethod
    def build_aspect_param_dict(cls, **kwargs):
        # return input parameters as a dict
        return kwargs
    
    @classmethod
    def get_method_aspect(cls, method_name):
        if cls not in CRAMS_ASPECT_DICT:
            return
        sz_aspect_obj = CRAMS_ASPECT_DICT.get(cls)
        if not isinstance(sz_aspect_obj, CramsAspect):
            raise exceptions.ValidationError('Crams Aspect for {} is not of type CramsAspect'.format(method_name))
        return sz_aspect_obj

    @classmethod
    def run_pre_aspect_for_method(cls, method_name, param_dict):
        sz_aspect_obj = cls.get_method_aspect(method_name)
        if not sz_aspect_obj:
            return
        if method_name not in sz_aspect_obj.pre_method_dict:
            return

        fn_list = sz_aspect_obj.pre_method_dict.get(method_name)
        for fn in fn_list:
            fn(cls, param_dict)

    @classmethod
    def run_post_aspect_for_method(cls, method_name, param_dict):
        sz_aspect_obj = cls.get_method_aspect(method_name)
        if not sz_aspect_obj:
            return
        if method_name not in sz_aspect_obj.post_method_dict:
            return

        fn_list = sz_aspect_obj.post_method_dict.get(method_name)
        for fn in fn_list:
            fn(cls, param_dict)


class CramsAPIActionStateModelSerializer(serializers.ModelSerializer, CramsAspectSerializer):
    """class ActionStateModelSerializer."""

    def __init__(self, instance=None, data=None, **kwargs):
        # No need to call this method, automatically called when serializer is intialised.
        if data:
            kwargs["data"] = data
        super().__init__(instance=instance, **kwargs)

        self.pp = pprint.PrettyPrinter(indent=4)  # For Debug
        self.cramsActionState = action_state.CramsAPIActionState(self)
        if self.cramsActionState.error_message:
            msg = 'ActionStateModelSerializer: ' + self.cramsActionState.error_message
            raise exceptions.ValidationError(msg)

        self.contact = contact_utils.fetch_user_contact_if_exists(self.get_current_user())

    def pprint(self, obj):
        self.pp.pprint(obj)

    def _setActionState(self):
        if not hasattr(self, 'cramsActionState'):
            self.cramsActionState = action_state.CramsAPIActionState(self)
            if self.cramsActionState.error_message:
                raise exceptions.ValidationError(
                    'ActionStateModelSerializer: ' +
                    self.cramsActionState.error_message)

    def get_current_user(self):
        user_obj = None
        if hasattr(self, 'cramsActionState'):
            user_obj = self.cramsActionState.rest_request.user
        if not user_obj:
            user_obj, _ = rest_utils.get_current_user_from_context(self)
        return user_obj

    def to_representation(self, instance):
        return super(CramsAPIActionStateModelSerializer, self).to_representation(instance)

    @classmethod
    def project_contact_has_readonly_access(cls, project, contact, erbs=None):
        # get erbs to check admin
        try:
            if erbs:
                if cls._is_admin(contact, erbs):
                    return False
            else:
                if project.requests.exists():
                    erbs = project.requests.first().e_research_system
                    if cls._is_admin(contact, erbs):
                        return False
        except:
            pass

        qs = project.project_contacts
        if qs and qs.filter(contact=contact,
                            contact_role__read_only=True).exists():
            return True
        return False

    def get_provision_roles_for_user(self, e_research_body_obj=None):
        fn = contact_utils.fetch_erb_userroles_with_provision_privileges
        return fn(self.get_current_user(), e_research_body_obj)

    def get_erbs_user_can_provision(self, e_research_body_obj=None):
        user_erb_roles = self.get_provision_roles_for_user(e_research_body_obj)
        erb_set = set()
        for erb_role in user_erb_roles:
            erb_set.add(erb_role.role_erb)
        return list(erb_set)

    def get_current_user_crams_token(self):
        return AbstractCramsRoleUtils.fetch_cramstoken_roles_dict(self.get_current_user())

    @staticmethod
    def _is_admin(user_contact, erbs):
        # check if user is a erb admin
        erb_admin_contacts = contact_models.CramsERBUserRoles.objects.filter(
            role_erb=erbs.e_research_body, contact=user_contact)

        # if user is an erb admin overrides the erbs admin
        for erb_admin_contact in erb_admin_contacts:
            if erb_admin_contact.is_erb_admin:
                return True

        # check if user is a erbs admin
        if erb_admin_contacts:
            if len(erb_admin_contacts) == 1:
                erb_admin_contact = erb_admin_contacts.first()
            else:
                # Fatal error users should have one or none CramsERBUserRole
                raise exceptions.ParseError(
                    'User has more than one CramsERBUserRole')

            # if user is an erb admin overrides the erbs admin
            if erb_admin_contact.is_erb_admin:
                return True

            # check user erbs admin matches with current erbs
            for admin_erbs in erb_admin_contact.admin_erb_systems.all():
                if admin_erbs == erbs:
                    return True

        return False



# coding=utf-8
"""
contact Serializers
"""
import logging

from crams.serializers import model_serializers
from crams.utils import role
from crams_log.constants import log_actions
from crams_log.models import CramsLog
from rest_framework import serializers, exceptions
from rest_framework.validators import UniqueValidator

from crams_contact import models as contact_models
from crams_contact.serializers import lookup_serializers, base_contact_serializer
from crams_contact.serializers.crams_role_init import CramsRoleInitUtils, CramsRoleInitMetaClass
from crams_contact.serializers.role_free_contact_serializer import RoleFreeContactDetailsLookupSerializer
from crams_contact.serializers.role_free_contact_serializer import RoleFreeContactSerializer
from crams_contact.serializers.contact_id_sz import ContactIDSerializer


LOG = logging.getLogger(__name__)


class ContactDetailSerializer(serializers.ModelSerializer):
    phone = serializers.CharField(required=False, allow_null=True)
    email = serializers.EmailField(required=False, allow_null=True)
    type = serializers.ChoiceField(choices=contact_models.ContactDetail.TYPE_CHOICES)
    restricted = serializers.BooleanField()

    class Meta(object):
        model = contact_models.ContactDetail


class CramsContactLookupSerializer(model_serializers.ModelLookupSerializer):
    """class Contact Lookup Serializer"""
    id = serializers.IntegerField(required=False)

    given_name = serializers.CharField(required=False)

    surname = serializers.CharField(required=False)

    email = serializers.EmailField(required=False)

    class Meta(object):
        """metaclass."""
        model = contact_models.Contact
        fields = ('id', 'given_name', 'surname', 'email')
        read_only_fields = ('given_name', 'surname', 'email')

    def validate(self, attrs):
        super().validate(attrs)
        self.verify_field_value('email', attrs, ignore_case=True)
        return self.instance or attrs


class CramsContactDetailsLookupSerializer(RoleFreeContactDetailsLookupSerializer,
                                          CramsRoleInitUtils, metaclass=CramsRoleInitMetaClass):
    email = serializers.EmailField(validators=[])

    phone = serializers.SerializerMethodField()

    organisation = lookup_serializers.OrganisationLookupSerializer(
        many=False, required=False, allow_null=True)

    contact_details = serializers.SerializerMethodField()

    contact_ids = serializers.SerializerMethodField()

    class Meta(object):
        model = contact_models.Contact
        fields = ('id', 'title', 'given_name', 'surname', 'email', 'phone',
                  'organisation', 'orcid', 'scopusid', 'contact_details',
                  'contact_ids', 'latest_contact')
        read_only_fields = ('title', 'given_name', 'surname', 'orcid',
                            'scopusid', 'phone', 'organisation',
                            'contact_details', 'id', 'latest_contact')
        pk_fields = ['email']

    def get_contact_ids(self, contact):
        if not contact:
            return list()
        self.role_init()
        if not contact == self.contact:
            if not self.user_erb_set:
                return list()
        if hasattr(contact, 'system_identifiers'):
            qs = contact.system_identifiers
            return ContactIDSerializer(qs, many=True).data

    def get_phone(self, contact):
        self.role_init()
        if self.contact == contact or self.user_erb_set:
            return super().get_phone(contact)

        # TODO review phone number visibility
        return super().get_phone(contact)

    def get_contact_details(self, contact):
        self.role_init()
        if self.contact == contact or self.user_erb_set:
            return super().get_contact_details(contact)

    def validate(self, attrs):
        if hasattr(self, 'initial_data'):
            print('CramsContactDetailsLookupSerializer', self.initial_data)
        else:
            print('CramsContactDetailsLookupSerializer inst', self.instance)
        return super().validate(attrs)


class CramsContactSerializer(RoleFreeContactSerializer,
                             CramsRoleInitUtils, metaclass=CramsRoleInitMetaClass):
    email = serializers.EmailField(
        required=True, validators=[
            UniqueValidator(queryset=contact_models.Contact.objects.all())])

    organisation = lookup_serializers.OrganisationLookupSerializer(
        many=False, required=False, allow_null=True)

    class Meta(object):
        model = contact_models.Contact
        fields = ('id', 'title', 'given_name', 'surname', 'email', 'phone',
                  'organisation', 'orcid', 'scopusid', 'contact_details',
                  'latest_contact', 'contact_ids')
        read_only_fields = ()
        pk_fields = ['email']

    def validate(self, data):
        current_user = self.get_current_user()
        if self.instance:
            if not current_user:
                user_msg = 'current user cannot be determined, update not permitted'
                raise exceptions.ValidationError(user_msg)

            contact_emails = self.get_related_emails(self.instance)
            if self.format_email(current_user.email) not in contact_emails:
                msg = 'Admin role required to update other Contact details'
                if not role.AbstractCramsRoleUtils.is_user_erb_admin(current_user):
                    raise exceptions.ValidationError(msg)

        return super().validate(data)


"""
from crams.models import User
qs = User.objects.all()
user_obj = qs.first()
from crams_contact.serializers import contact_serializer
sz_class = contact_serializer.CramsContactDetailsLookupSerializer
c = sz_class.fetch_or_create_given_user_as_contact(user_obj)

"""


class ContactSerializer(base_contact_serializer.BaseContactSerializer):
    email = serializers.EmailField(
        required=True, validators=[
            UniqueValidator(queryset=contact_models.Contact.objects.all())])

    contact_ids = serializers.SerializerMethodField()

    organisation = lookup_serializers.OrganisationLookupSerializer(
        many=False, required=False, allow_null=True)

    last_login = serializers.SerializerMethodField()

    class Meta(object):
        model = contact_models.Contact
        fields = ['id', 'title', 'given_name', 'surname', 'email', 'phone',
                  'organisation', 'orcid', 'scopusid', 'contact_details',
                  'latest_contact', 'contact_ids', 'last_login']
        read_only_fields = []

    @classmethod
    def get_last_login(cls, obj):
        qs = CramsLog.objects.filter(
            created_by__iexact=obj.email,
            action__action_type=log_actions.LOGIN).order_by('-creation_ts')

        # distinct get one of each site logged in
        sites = qs.order_by('type').values_list('type__name').distinct()
        results = []
        try:
            for site in sites:
                qs_sites = qs.filter(type__name=site[0])
                if qs_sites:
                    results.append({'site': qs_sites[0].type.name, 'date': qs_sites[0].creation_ts})
        except:
            return None

        return results

    @classmethod
    def add_related_entities(cls, contact, validated_data):
        RoleFreeContactSerializer.add_related_entities(contact, validated_data)

    def validate(self, data):
        self._setActionState()
        # the following fields are serializer method fields
        data['phone'] = self.initial_data.get('phone')

        contact_details = self.initial_data.get('contact_details')
        if contact_details:
            self.validate_contact_details(contact_details)
            data['contact_details'] = contact_details

        if self.cramsActionState.is_update_action:
            current_user = self.get_current_user()
            instance = self.cramsActionState.existing_instance
            if not role.AbstractCramsRoleUtils.is_user_erb_admin(current_user):
                contact_emails = self.get_related_emails(instance)
                msg = 'Admin role required to update other Contact details'
                if self.format_email(current_user.email) not in contact_emails:
                    raise exceptions.ValidationError(msg)

        if self.cramsActionState.is_create_action:
            latest_contact = data.get('latest_contact')
            if latest_contact:
                msg = 'historical Contact, edit current contact instead'
                raise exceptions.ValidationError(msg)

        return data

    @classmethod
    def setup_default_contact_detail(cls, parent_contact):
        return RoleFreeContactSerializer.setup_default_contact_detail(parent_contact=parent_contact)

    def create(self, validated_data):
        print(' in create proper')
        return RoleFreeContactSerializer.create(validated_data=validated_data)

    def update(self, instance, validated_data):
        return RoleFreeContactSerializer.update(instance, validated_data)



# coding=utf-8
"""

"""

# Create your views here.

from rest_framework import filters
from rest_framework.decorators import action
from rest_framework.response import Response

from crams import permissions
from crams.utils import django_utils
from crams_contact.models import Contact
from crams_contact.serializers import contact_serializer
from crams_contact.serializers.base_contact_serializer import BaseContactSerializer
from crams_collection.serializers.contact_project_list import ContactProjectListSerializer


class ContactViewSet(django_utils.CramsModelViewSet):
    """
    class ContactViewSet
    """
    permission_classes = [permissions.IsCramsAuthenticated]
    queryset = Contact.objects.all()
    serializer_class = contact_serializer.ContactSerializer
    filter_backends = (filters.SearchFilter,)
    search_fields = ('given_name', 'surname', 'email')

    @action(detail=False, url_path='(?P<email>[^@]+@[^@]+.[^@\d.]+)')
    def search_by_email(self, request, email=None):
        if email:
            context = {'request': request}
            queryset = self.serializer_class.search_contact_get_queryset(email=email)
            serializer = self.serializer_class(queryset, many=True, context=context)
            return Response(serializer.data)
        return Response(dict())


class AdminContactViewSet(ContactViewSet):
    """
     class Admin ContactViewSet
     """
    permission_classes = [permissions.IsCramsAuthenticated,
                          permissions.IsCramsAdmin]
    queryset = Contact.objects.all()
    serializer_class = BaseContactSerializer

    @action(detail=False, url_path='(?P<contact_id>\d+)/list/project')
    def get_contact_projects(self, request, contact_id):
        context = {'request': request}
        queryset = self.serializer_class.search_contact_get_queryset(pk=contact_id)
        serializer = ContactProjectListSerializer(
            queryset.first(), context=context)
        return Response(serializer.data)



# coding=utf-8
"""
read only contact Serializer
"""
import logging

from django.db.models import Q
from rest_framework import serializers, exceptions

from crams.utils import rest_utils
from crams.models import User
from crams_contact import models as contact_models
from crams_contact.serializers import contact_id_sz
from crams_contact.serializers import lookup_serializers
from crams_contact.serializers.role_free_contact_serializer import RoleFreeContactSerializer
from crams_contact.serializers.util_sz import CramsAPIActionStateModelSerializer
from crams_contact.utils import contact_utils

LOG = logging.getLogger(__name__)


def get_serializer_user_contact(serializer_self):
    user, context = rest_utils.get_current_user_from_context(serializer_self)
    if user:
        return BaseContactSerializer(context=context). \
            fetch_or_create_given_user_as_contact(user)

    msg = '"context" object not found, required to identify current user.'
    raise exceptions.ValidationError({'message': msg})


class BaseContactSerializer(CramsAPIActionStateModelSerializer):
    """
    A read-only version of contact with limited ability to add new contact for search purposes
    - allows creation of a new contact with email value only.
    """
    email = serializers.EmailField(validators=[])

    phone = serializers.SerializerMethodField()

    organisation = lookup_serializers.OrganisationLookupSerializer(
        many=False, required=False, allow_null=True)

    contact_details = serializers.SerializerMethodField()

    class Meta(object):
        model = contact_models.Contact
        fields = ['id', 'title', 'given_name', 'surname', 'email', 'phone',
                  'organisation', 'orcid', 'scopusid', 'contact_details',
                  'latest_contact']
        read_only_fields = ['title', 'given_name', 'surname', 'orcid',
                            'scopusid', 'phone', 'organisation',
                            'contact_details', 'id', 'latest_contact']

    def update(self, instance, validated_data):
        raise exceptions.ParseError('Update not allowed ')

    def __delete__(self, instance):
        raise exceptions.ParseError('Delete not allowed ')

    def create(self, validated_data):
        print('in Base')
        raise exceptions.ParseError('Create not allowed ')

    @classmethod
    def get_details_queryset(cls, contact):
        qs = contact.contact_details
        return qs.filter(restricted=False).order_by('type')

    def intialise(self):
        if not hasattr(self, 'self_contact'):
            self.self_contact = get_serializer_user_contact(self)

            self.user_erb_system_list = \
                contact_utils.fetch_related_user_erb_system_list(self)

            self.user_erb_set = set()
            for erb_system in self.user_erb_system_list:
                self.user_erb_set.add(erb_system.e_research_body)

    @classmethod
    def get_email(cls, contact):
        if not contact.email:
            for detail in cls.get_details_queryset(contact):
                if detail.email:
                    return detail.email
        return contact.email

    @classmethod
    def get_organisation(cls, contact):
        if contact.organisation:
            return contact.organisation
        else:
            return None

    @classmethod
    def search_contact_get_queryset(cls, pk=None, email=None):
        search_dict = dict()
        if pk:
            search_dict['id'] = pk
        if email:
            search_dict['email'] = email
        if search_dict:
            return RoleFreeContactSerializer.fetch_contacts(search_dict)
        return None

    def get_contact_ids(self, contact_obj):
        self.intialise()
        if not contact_obj:
            return list()

        qs = contact_obj.system_identifiers
        if not contact_obj == self.self_contact:
            if not self.user_erb_set:
                return list()
            else:
                # if not self, should see only authorised erb identifiers
                filter_qs = Q(system__e_research_body__in=self.user_erb_set)
                qs = qs.filter(filter_qs)

        return contact_id_sz.ContactIDSerializer(qs, many=True).data

    def get_phone(self, contact):
        def get_first_available_phone_number():
            if not contact.phone:
                for detail in self.get_details_queryset(contact):
                    phone = detail.phone
                    if phone:
                        return phone
            return contact.phone

        self.intialise()
        if self.self_contact == contact or self.user_erb_set:
            return get_first_available_phone_number()

        # TODO review phone number visibility
        return get_first_available_phone_number()

    def get_contact_details(self, contact):
        self.intialise()
        if self.self_contact == contact or self.user_erb_set:
            ret_list = []
            for detail in self.get_details_queryset(contact):
                detail_dict = dict()
                detail_dict['type'] = detail.get_type_display()
                detail_dict['phone'] = detail.phone
                detail_dict['email'] = detail.email
                ret_list.append(detail_dict)
            return ret_list

    @classmethod
    def get_related_emails(cls, contact_obj):
        return RoleFreeContactSerializer.get_related_emails(contact_obj)

    @classmethod
    def format_email(cls, email_in):
        if email_in:
            return email_in.strip().lower()
        return None

    @classmethod
    def validate_email(cls, email):
        email = cls.format_email(email)
        if not email:
            msg = 'Email required, cannot resolve to None'
            raise exceptions.ValidationError(msg)
        return email

    @classmethod
    def validate_contact_details(cls, contact_details):
        if contact_details:
            for c_d in contact_details:
                c_d['email'] = cls.format_email(c_d.get('email'))
        return contact_details

    def validate(self, data):
        msg = 'email required to fetch contact data'
        email = data.get('email')
        try:
            if email:
                return contact_models.Contact.objects.get(email__iexact=email)
        except contact_models.Contact.DoesNotExist:
            msg = 'Contact not found for email ' + email
        except Exception as e:
            msg = str(e)

        raise exceptions.ValidationError(msg)

    def fetch_or_create_given_user_as_contact(self, user_obj):
        if not isinstance(user_obj, User):
            msg = 'Contact fetch expects User object, got'
            raise exceptions.ValidationError(msg + type(user_obj))

        contact_data = {
            'email': user_obj.email,
            'given_name': user_obj.first_name,
            'surname': user_obj.last_name
        }
        return self.get_or_create(contact_data)

    def get_or_create(self, validated_data):
        contact = None
        try:
            contact = RoleFreeContactSerializer.fetch_first_contact_obj(validated_data=validated_data)
        except contact_models.Contact.DoesNotExist:
            pass
        if not contact:
            contact = self.create(validated_data)
        return contact



# coding=utf-8
"""

"""
import logging
from rest_framework import serializers
from crams.models import User
from crams.utils import rest_utils
from crams.utils.role import AbstractCramsRoleUtils
from rest_framework import exceptions

from crams_contact import models as contact_models

LOG = logging.getLogger(__name__)


class CramsRoleInitMetaClass(serializers.SerializerMetaclass):
    def __init__(cls, class_name, superclasses, attribute_dict):
        super().__init__(class_name, superclasses, attribute_dict)
        cls.current_user = None
        cls.contact = None
        cls.user_erb_set = set()
        cls.user_erb_system_list = list()

    # def __new__(mcs, class_name, bases, dct):
    #     new_class = super(CramsRoleInitMetaClass, mcs).__new__(mcs, class_name, bases, dct)
    #     setattr(new_class, mcs.role_init.__name__, mcs.role_init)
    #     setattr(new_class, mcs.get_current_user.__name__, mcs.get_current_user)
    #     setattr(new_class, mcs.fetch_or_create_given_user_as_contact.__name__,
    #             mcs.fetch_or_create_given_user_as_contact)
    #     return new_class


class CramsRoleInitUtils:
    # Note: The init below is not called by the Model Serializer
    #  - so we use the above CramsRoleInitMetaClass to initialize the variables
    #  - the init below is dummy and never used when inherited together with Model Serializer
    def __init__(self):
        self.current_user = None
        self.contact = None
        self.user_erb_set = set()
        self.user_erb_system_list = list()

    def role_init(self):
        if not self.contact:
            user_obj = self.get_current_user()
            if user_obj:
                self.contact = self.fetch_or_create_given_user_as_contact(user_obj)
                self.user_erb_system_list = AbstractCramsRoleUtils.get_authorised_e_research_system_list(user_obj)
                for erb_system in self.user_erb_system_list:
                    self.user_erb_set.add(erb_system.e_research_body)

    @classmethod
    def fetch_or_create_given_user_as_contact(cls, user_obj):
        if not isinstance(user_obj, User):
            msg = 'Contact fetch expects User object, got'
            raise exceptions.ValidationError(msg + type(user_obj))
        try:
            contact = contact_models.Contact.objects.get(email=user_obj.email)
        except contact_models.Contact.DoesNotExist:
            contact_data = {
                'email': user_obj.email,
                'given_name': user_obj.first_name,
                'surname': user_obj.last_name
            }
            contact = contact_models.Contact(**contact_data)
            contact.save()
        return contact

    def get_current_user(self):
        if not self.current_user:
            self.current_user, context = rest_utils.get_current_user_from_context(self)
        return self.current_user



# coding=utf-8
"""

"""
from json import dumps as json_dumps

from crams.models import CramsToken
from crams.utils import role

from crams_contact.models import CramsERBUserRoles
from crams_contact.utils import contact_utils

# Crams Role dict Keys
ADMIN_ROLE_KEY = 'admin'
APPROVER_ROLE_KEY = 'approver'
PROVIDER_ROLE_KEY = 'provisioner'
DELEGATE_ROLE_TYPE = 'delegate'
TENANT_MANAGER = 'tenant_manager'
SERVICE_MANAGEMENT_ROLE_TYPE = 'service_management'
FACULTY_MANAGEMENT_ROLE_TYPE = 'faculty_management'
LEGACY_ROLES_KEY = 'global'


class ContactErbRoleSerializer(role.AbstractCramsRoleUtils):
    @classmethod
    def setup_crams_token_and_roles(cls, user_obj):
        crams_token, created = CramsToken.objects.get_or_create(user=user_obj)
        if crams_token.is_expired():  # Expire existing Token
            crams_token.delete()
            crams_token = CramsToken(user=user_obj)

        crams_token.ks_roles = json_dumps(cls.build_user_roles(user_obj))
        crams_token.save()
        return crams_token

    @classmethod
    def build_user_roles(cls, user_obj):
        contact = contact_utils.fetch_contact_for_user(user_obj)
        if not contact:
            return dict()
        return cls.build_contact_roles(contact)

    @classmethod
    def build_contact_roles(cls, contact):
        def update_roles_for_erb_system(erb_system_obj):
            approver_set.add(erb_system_obj.approver_role)
            if hasattr(erb_system_obj, 'storageproduct'):
                for sp in erb_system_obj.storageproduct.all():
                    provider_set.add(sp.provider.provider_role)
            if hasattr(erb_system_obj, 'computeproduct'):
                for cp in erb_system_obj.computeproduct.all():
                    provider_set.add(cp.provider.provider_role)

        roles_dict = dict()
        if not contact:
            return roles_dict

        admin_set = set()
        approver_set = set()
        provider_set = set()

        current_erb_roles = CramsERBUserRoles.query_current_roles(contact)
        if current_erb_roles.exists():
            for erb_user_role in current_erb_roles.filter(
                    end_date_ts__isnull=True):
                if erb_user_role.is_erb_admin:
                    admin_set.add(erb_user_role.role_erb.admin_role)
                    for erb_system in erb_user_role.role_erb.systems.all():
                        update_roles_for_erb_system(erb_system)

                if erb_user_role.admin_erb_systems.exists():
                    for erb_system in erb_user_role.admin_erb_systems.all():
                        admin_set.add(erb_system.admin_role)
                        update_roles_for_erb_system(erb_system)

                if erb_user_role.approver_erb_systems.exists():
                    for erb_system in erb_user_role.approver_erb_systems.all():
                        approver_set.add(erb_system.approver_role)

                if erb_user_role.delegates.exists():
                    for delegate in erb_user_role.delegates.all():
                        approver_set.add(delegate.approver_role)

                if erb_user_role.providers.exists():
                    for provider in erb_user_role.providers.all():
                        provider_set.add(provider.provider_role)

        if admin_set:
            roles_dict[ADMIN_ROLE_KEY] = list(admin_set)
        if approver_set:
            roles_dict[APPROVER_ROLE_KEY] = list(approver_set)
        if provider_set:
            roles_dict[PROVIDER_ROLE_KEY] = list(provider_set)

        if hasattr(contact, 'servicemanager') and contact.servicemanager.active:
            roles_dict[SERVICE_MANAGEMENT_ROLE_TYPE] = [
                contact.servicemanager.manager_role]

        manager_roles = set()
        if hasattr(contact, 'manager_faculties'):
            for faculty_manager in contact.manager_faculties.filter(active=True):
                manager_roles.add(faculty_manager.faculty.manager_role)
        if hasattr(contact, 'manager_organisations'):
            for org_manager in contact.manager_organisations.filter(active=True):
                for faculty in org_manager.organisation.faculties.all():
                    manager_roles.add(faculty.manager_role)
        if manager_roles:
            roles_dict[FACULTY_MANAGEMENT_ROLE_TYPE] = list(manager_roles)

        return roles_dict



from django.db import models
from rest_framework import exceptions

# Create your models here.

from crams import models as crams_models
from crams_contact.utils import validators
from crams_log.models import AbstractLinkToCramsLog


class Organisation(models.Model):
    name = models.CharField(max_length=200)
    short_name = models.CharField(max_length=50, db_index=True)
    ands_url = models.URLField(null=True, blank=True)
    notification_email = models.EmailField()

    class Meta:
        app_label = 'crams_contact'

    def __str__(self):  # __unicode__ on Python 2
        return self.__unicode__()

    def __unicode__(self):
        return str(self.short_name)

    @property
    def manager_role(self):
        return 'pk_{}.organisation_manager'.format(self.id)


class Faculty(models.Model):
    faculty_code = models.CharField(max_length=200)
    name = models.CharField(max_length=200)
    organisation = models.ForeignKey(Organisation, related_name='faculties', on_delete=models.DO_NOTHING)

    class Meta:
        app_label = 'crams_contact'

    def __str__(self):  # __unicode__ on Python 2
        return self.__unicode__()

    def __unicode__(self):
        return '{} / {}'.format(self.name, self.organisation)

    @property
    def manager_role(self):
        return 'pk_{}.faculty_manager'.format(self.id)


class Department(models.Model):
    department_code = models.CharField(max_length=200)
    name = models.CharField(max_length=200)
    faculty = models.ForeignKey(Faculty, related_name='departments', on_delete=models.DO_NOTHING)

    class Meta:
        app_label = 'crams_contact'

    def __str__(self):  # __unicode__ on Python 2
        return self.__unicode__()

    def __unicode__(self):
        return '{} / {}'.format(self.name, self.faculty)

    @property
    def manager_role(self):
        return 'pk_{}.department_manager'.format(self.id)


class ContactRole(models.Model):
    """
    ContactRole Model
    """
    name = models.CharField(max_length=100, unique=True)

    e_research_body = models.ForeignKey(crams_models.EResearchBody, blank=True, null=True,
                                        related_name='contact_roles',
                                        default=None, on_delete=models.DO_NOTHING)

    project_leader = models.BooleanField(default=False)

    read_only = models.BooleanField(default=False)

    support_notification = models.BooleanField(default=False)

    class Meta:
        app_label = 'crams_contact'
        unique_together = ('name', 'e_research_body')

    def __str__(self):
        return '{}'.format(self.name)


class Contact(models.Model):
    """
    Contact Model
    """
    title = models.CharField(
        max_length=50, blank=True, null=True
    )

    given_name = models.CharField(
        max_length=200, blank=True, null=True
    )

    surname = models.CharField(
        max_length=200, blank=True, null=True
    )

    email = models.EmailField(unique=True, db_index=True)

    phone = models.CharField(
        max_length=50, blank=True, null=True
    )

    organisation = models.ForeignKey(Organisation,
                                     blank=True, null=True, on_delete=models.DO_NOTHING)

    orcid = models.URLField(blank=True, null=True,
                            validators=[validators.validate_orcid],
                            verbose_name='ORCID of contact')

    scopusid = models.CharField(
        max_length=50, blank=True, null=True
    )

    latest_contact = models.ForeignKey('Contact', blank=True, null=True, on_delete=models.DO_NOTHING)

    class Meta:
        app_label = 'crams_contact'

    def __str__(self):
        return '{0} {1} {2} {3}'.format(
            self.given_name, self.surname, self.title, self.pk)

    def get_full_name(self):
        def fmt(in_str):
            if in_str:
                return in_str + ' '
            return ''

        return fmt(self.title) + fmt(self.given_name) + fmt(self.surname)

    @property
    def system_identifiers(self):
        return EResearchContactIdentifier.objects.filter(
            contact=self, parent_erb_contact_id__isnull=True)

    @classmethod
    def fetch_contact_for_user(cls, user_obj):
        if isinstance(user_obj, crams_models.User):
            qs = Contact.objects.filter(
                email__iexact=user_obj.email, latest_contact__isnull=True)
            if qs.exists():
                return qs.first()
        return None

    def fetch_user_for_contact(self):
        return crams_models.User.objects.get(email=self.email)


class ContactDetail(models.Model):
    BUSINESS = 'B'
    MOBILE = 'M'
    PERSONAL = 'P'
    TYPE_CHOICES = ((BUSINESS, 'Business'), (PERSONAL, 'Personal'),
                    (MOBILE, 'Mobile'))
    type = models.CharField(
        max_length=1, choices=TYPE_CHOICES, default=PERSONAL)
    parent_contact = models.ForeignKey(Contact, related_name='contact_details', on_delete=models.DO_NOTHING)
    restricted = models.BooleanField(default=False)
    email = models.EmailField(blank=True, null=True)
    phone = models.CharField(max_length=50, blank=True, null=True)

    class Meta:
        app_label = 'crams_contact'

    @classmethod
    def get_choice_value(cls, choice_str):
        if choice_str:
            for k, v in ContactDetail.TYPE_CHOICES:
                if choice_str.strip().capitalize() == v:
                    return k
        return None


class EResearchContactIdentifier(crams_models.ProvisionableItem):
    identifier = models.CharField(max_length=254)

    contact = models.ForeignKey(Contact, related_name='archive_contact_ids', on_delete=models.DO_NOTHING)

    system = models.ForeignKey(
        crams_models.EResearchBodyIDKey, related_name='contact_identifiers', on_delete=models.DO_NOTHING)

    parent_erb_contact_id = models.ForeignKey(
        'EResearchContactIdentifier', blank=True, null=True, on_delete=models.DO_NOTHING)

    class Meta:
        app_label = 'crams_contact'

    def __str__(self):
        return '{} / {} - {}'.format(
            self.identifier, self.contact, self.system)

    @classmethod
    def get_provider(cls):
        return None


class ContactLabel(crams_models.CramsCommon):
    label_name = models.CharField(max_length=255)

    active = models.BooleanField(default=True)

    contact = models.ForeignKey(Contact, related_name="labels", on_delete=models.CASCADE)

    class Meta:
        app_label = 'crams_contact'
        unique_together = ('label_name', 'contact')

    def __str__(self):
        return '{} / {}'.format(self.label_name, self.contact.get_full_name())


class CramsERBUserRoles(models.Model):
    contact = models.ForeignKey(Contact, related_name='user_roles', on_delete=models.DO_NOTHING)

    role_erb = models.ForeignKey(crams_models.EResearchBody, related_name='contacts', on_delete=models.DO_NOTHING)

    is_erb_admin = models.BooleanField(default=False)

    is_tenant_manager = models.BooleanField(default=False)

    delegates = models.ManyToManyField(
        crams_models.EResearchBodyDelegate, related_name='delegate_contacts')

    admin_erb_systems = models.ManyToManyField(
        crams_models.EResearchBodySystem, related_name='admin_contacts')

    approver_erb_systems = models.ManyToManyField(
        crams_models.EResearchBodySystem, related_name='approver_contacts')

    providers = models.ManyToManyField(
        crams_models.Provider, related_name='provider_contacts')

    end_date_ts = models.DateTimeField(blank=True, null=True)

    class Meta:
        app_label = 'crams_contact'
        unique_together = ('contact', 'role_erb', 'end_date_ts')

    def __str__(self):
        return self.contact.email

    @classmethod
    def query_current_roles(cls, contact_obj, erb_obj=None):
        qs_filter = models.Q(contact=contact_obj, end_date_ts__isnull=True)
        if erb_obj:
            qs_filter &= models.Q(role_erb=erb_obj)
        return CramsERBUserRoles.objects.filter(qs_filter)


class ContactLog(AbstractLinkToCramsLog):
    crams_contact_db_id = models.IntegerField(blank=True, null=True)

    class Meta:
        app_label = 'crams_contact'
        unique_together = ('crams_contact_db_id', 'log_parent')

    def __str__(self):
        return '{} {}'.format(self.log_parent, self.crams_contact_db_id)

    @classmethod
    def link_log(cls, log_obj, crams_contact_obj):
        if not isinstance(crams_contact_obj, Contact):
            exceptions.ValidationError('Contact object required to link to ContactLog')
        crams_contact_db_id = crams_contact_obj.id
        obj, _ = cls.objects.get_or_create(log_parent=log_obj, crams_contact_db_id=crams_contact_db_id)
        return obj

    @classmethod
    def fetch_log_qs(cls, crams_contact_db_id):
        return cls.objects.filter(crams_contact_db_id=crams_contact_db_id)



# coding=utf-8
"""
Crams Permissions
"""

from rest_framework import permissions

from crams_contact.serializers.contact_erb_roles import ContactErbRoleSerializer


class IsServiceManager(permissions.BasePermission):
    """
    Global permission to allow Service Manager role
    """
    message = 'User does not hold Service Manager role.'

    @classmethod
    def role_exists_for_user(cls, user):
        user_roles_dict = ContactErbRoleSerializer.fetch_cramstoken_roles_dict(user)
        if user_roles_dict:
            return ContactErbRoleSerializer.SERVICE_MANAGEMENT_ROLE_TYPE in user_roles_dict
        return False

    def has_permission(self, request, view):
        """
        user has Service Manager role
        :param request:
        :param view:
        :return:
        """
        return self.role_exists_for_user(request.user)



from django.db import models
from crams_contact import models as contact_model


class ServiceManager(models.Model):
    contact = models.OneToOneField(contact_model.Contact, unique=True, on_delete=models.DO_NOTHING)

    active = models.BooleanField(default=True)

    class Meta(object):
        app_label = 'crams_manager'

    def __str__(self):
        return '{}'.format(self.contact)

    @property
    def manager_role(self):
        return 'service_manager'


class FacultyManager(models.Model):
    contact = models.ForeignKey(
        contact_model.Contact, related_name='manager_faculties', on_delete=models.DO_NOTHING)

    faculty = models.ForeignKey(
        contact_model.Faculty, related_name='faculty_managers', on_delete=models.DO_NOTHING)

    active = models.BooleanField(default=True)

    class Meta(object):
        app_label = 'crams_manager'
        unique_together = ('contact', 'faculty')

    def __str__(self):
        return '{} - {}'.format(self.contact, self.faculty)


class DepartmentManager(models.Model):
    contact = models.ForeignKey(
        contact_model.Contact, related_name='manager_departments', on_delete=models.DO_NOTHING)

    department = models.ForeignKey(
        contact_model.Department, related_name='department_managers', on_delete=models.DO_NOTHING)

    active = models.BooleanField(default=True)

    class Meta(object):
        app_label = 'crams_manager'
        unique_together = ('contact', 'department')

    def __str__(self):
        return '{} - {}'.format(self.contact, self.department)


class OrganisationManager(models.Model):
    contact = models.ForeignKey(
        contact_model.Contact, related_name='manager_organisations', on_delete=models.DO_NOTHING)

    organisation = models.ForeignKey(
        contact_model.Organisation, related_name='organisation_managers', on_delete=models.DO_NOTHING)

    active = models.BooleanField(default=True)

    class Meta(object):
        app_label = 'crams_manager'
        unique_together = ('contact', 'organisation')

    def __str__(self):
        return '{} - {}'.format(self.contact, self.organisation)
